@@ -30,12 +30,12 @@
 #include "certsave.h"
 #include "cm.h"
 #include "csrgen.h"
+#include "hook.h"
 #include "iterate.h"
 #include "keygen.h"
 #include "keyiread.h"
 #include "log.h"
 #include "notify.h"
-#include "postsave.h"
 #include "prefs.h"
 #include "store.h"
 #include "store-int.h"
@@ -48,7 +48,7 @@ struct cm_iterate_state {
 	struct cm_csrgen_state *cm_csrgen_state;
 	struct cm_submit_state *cm_submit_state;
 	struct cm_certsave_state *cm_certsave_state;
-	struct cm_postsave_state *cm_postsave_state;
+	struct cm_hook_state *cm_hook_state;
 	struct cm_certread_state *cm_certread_state;
 	struct cm_notify_state *cm_notify_state;
 };
@@ -107,6 +107,12 @@ cm_entry_reset_state(struct cm_store_entry *entry)
 		break;
 	case CM_NEED_TO_SAVE_CERT:
 		break;
+	case CM_START_SAVING_CERT:
+		entry->cm_state = CM_NEED_TO_SAVE_CERT;
+		break;
+	case CM_PRE_SAVE_CERT:
+		entry->cm_state = CM_NEED_TO_SAVE_CERT;
+		break;
 	case CM_SAVING_CERT:
 		entry->cm_state = CM_NEED_TO_SAVE_CERT;
 		break;
@@ -261,7 +267,10 @@ cm_iterate_init(struct cm_store_entry *entry, void **cm_iterate_state)
 
 /* Check if the entry's expiration has crossed an interesting threshold. */
 static int
-cm_check_expiration_is_noteworthy(struct cm_store_entry *entry)
+cm_check_expiration_is_noteworthy(struct cm_store_entry *entry,
+				  int (*get_ttls)(const time_t **,
+						  unsigned int *),
+				  time_t *last_check)
 {
 	unsigned int i, n_ttls;
 	time_t now, ttl, previous_ttl;
@@ -282,18 +291,17 @@ cm_check_expiration_is_noteworthy(struct cm_store_entry *entry)
 		ttl = entry->cm_cert_not_after - now;
 	}
 	/* How much time was left, last time we checked? */
-	if (entry->cm_cert_not_after < entry->cm_last_expiration_check) {
+	if (entry->cm_cert_not_after < *last_check) {
 		previous_ttl = 0;
 	} else {
-		previous_ttl = entry->cm_cert_not_after -
-			       entry->cm_last_expiration_check;
+		previous_ttl = entry->cm_cert_not_after - *last_check;
 	}
 	/* Note that we're checking now. */
-	entry->cm_last_expiration_check = now;
+	*last_check = now;
 	/* Which list of interesting values are we consulting? */
 	ttls = NULL;
 	n_ttls = 0;
-	if ((cm_prefs_ttls(&ttls, &n_ttls) != 0) || (n_ttls == 0)) {
+	if (((*get_ttls)(&ttls, &n_ttls) != 0) || (n_ttls == 0)) {
 		return -1;
 	}
 	/* Check for crosses. */
@@ -653,26 +661,37 @@ cm_iterate(struct cm_store_entry *entry, struct cm_store_ca *ca,
 			if (cm_submit_unreachable(entry,
 						  state->cm_submit_state) == 0) {
 				/* Let's try again later. */
+				*delay = cm_submit_specified_delay(entry,
+								   state->cm_submit_state);
 				cm_submit_done(entry, state->cm_submit_state);
 				state->cm_submit_state = NULL;
 				entry->cm_state = CM_CA_UNREACHABLE;
 				*when = cm_time_delay;
-				*delay = cm_decide_ca_delay(remaining);
+				if (*delay < 0) {
+					*delay = cm_decide_ca_delay(remaining);
+				}
 			} else
 			if (cm_submit_save_ca_cookie(entry,
 						     state->cm_submit_state) == 0) {
 				/* Saved CA's identifier for our request; give
-				 * it a little time and then ask. */
+				 * it the specified time, or a little time, and
+				 * then ask for a progress update. */
+				*delay = cm_submit_specified_delay(entry,
+								   state->cm_submit_state);
 				cm_submit_done(entry, state->cm_submit_state);
 				state->cm_submit_state = NULL;
 				entry->cm_state = CM_CA_WORKING;
 				*when = cm_time_delay;
-				*delay = cm_decide_ca_delay(remaining);
+				if (*delay < 0) {
+					*delay = cm_decide_ca_delay(remaining);
+				}
 			} else
 			if (cm_submit_unconfigured(entry,
 						   state->cm_submit_state) == 0) {
 				/* Saved CA's identifier for our request; give
 				 * it a little time and then ask. */
+				*delay = cm_submit_specified_delay(entry,
+								   state->cm_submit_state);
 				cm_submit_done(entry, state->cm_submit_state);
 				state->cm_submit_state = NULL;
 				if (entry->cm_cert != NULL) {
@@ -686,7 +705,9 @@ cm_iterate(struct cm_store_entry *entry, struct cm_store_ca *ca,
 				} else {
 					entry->cm_state = CM_CA_UNCONFIGURED;
 					*when = cm_time_delay;
-					*delay = cm_decide_ca_delay(remaining);
+					if (*delay < 0) {
+						*delay = cm_decide_ca_delay(remaining);
+					}
 				}
 			} else {
 				/* Don't know what's going on. HELP! */
@@ -713,6 +734,49 @@ cm_iterate(struct cm_store_entry *entry, struct cm_store_ca *ca,
 		break;
 
 	case CM_NEED_TO_SAVE_CERT:
+		if (entry->cm_pre_certsave_command != NULL) {
+			state->cm_hook_state = cm_hook_start_presave(entry);
+			if (state->cm_hook_state != NULL) {
+				/* Note that we're doing the pre-save. */
+				entry->cm_state = CM_PRE_SAVE_CERT;
+				/* Wait for status update, or poll. */
+				*readfd = cm_hook_get_fd(entry,
+							 state->cm_hook_state);
+				if (*readfd == -1) {
+					*when = cm_time_soon;
+				} else {
+					*when = cm_time_no_time;
+				}
+			} else {
+				/* Failed to start the pre-save; skip it. */
+				entry->cm_state = CM_START_SAVING_CERT;
+				*when = cm_time_now;
+			}
+		} else {
+			entry->cm_state = CM_START_SAVING_CERT;
+			*when = cm_time_now;
+		}
+		break;
+
+	case CM_PRE_SAVE_CERT:
+		if (cm_hook_ready(entry, state->cm_hook_state) == 0) {
+			cm_hook_done(entry, state->cm_hook_state);
+			state->cm_hook_state = NULL;
+			entry->cm_state = CM_START_SAVING_CERT;
+			*when = cm_time_now;
+		} else {
+			/* Wait for status update, or poll. */
+			*readfd = cm_hook_get_fd(entry,
+						 state->cm_hook_state);
+			if (*readfd == -1) {
+				*when = cm_time_soon;
+			} else {
+				*when = cm_time_no_time;
+			}
+		}
+		break;
+
+	case CM_START_SAVING_CERT:
 		state->cm_certsave_state = cm_certsave_start(entry);
 		if (state->cm_certsave_state != NULL) {
 			/* Note that we're saving the cert. */
@@ -805,13 +869,13 @@ cm_iterate(struct cm_store_entry *entry, struct cm_store_ca *ca,
 
 	case CM_SAVED_CERT:
 		if (entry->cm_post_certsave_command != NULL) {
-			state->cm_postsave_state = cm_postsave_start(entry);
-			if (state->cm_postsave_state != NULL) {
+			state->cm_hook_state = cm_hook_start_postsave(entry);
+			if (state->cm_hook_state != NULL) {
 				/* Note that we're doing the post-save. */
 				entry->cm_state = CM_POST_SAVED_CERT;
 				/* Wait for status update, or poll. */
-				*readfd = cm_postsave_get_fd(entry,
-							     state->cm_postsave_state);
+				*readfd = cm_hook_get_fd(entry,
+							 state->cm_hook_state);
 				if (*readfd == -1) {
 					*when = cm_time_soon;
 				} else {
@@ -829,15 +893,15 @@ cm_iterate(struct cm_store_entry *entry, struct cm_store_ca *ca,
 		break;
 
 	case CM_POST_SAVED_CERT:
-		if (cm_postsave_ready(entry, state->cm_postsave_state) == 0) {
-			cm_postsave_done(entry, state->cm_postsave_state);
-			state->cm_postsave_state = NULL;
+		if (cm_hook_ready(entry, state->cm_hook_state) == 0) {
+			cm_hook_done(entry, state->cm_hook_state);
+			state->cm_hook_state = NULL;
 			entry->cm_state = CM_MONITORING;
 			*when = cm_time_now;
 		} else {
 			/* Wait for status update, or poll. */
-			*readfd = cm_postsave_get_fd(entry,
-						     state->cm_postsave_state);
+			*readfd = cm_hook_get_fd(entry,
+						 state->cm_hook_state);
 			if (*readfd == -1) {
 				*when = cm_time_soon;
 			} else {
@@ -874,12 +938,25 @@ cm_iterate(struct cm_store_entry *entry, struct cm_store_ca *ca,
 
 	case CM_MONITORING:
 		if (entry->cm_monitor &&
-		    (cm_check_expiration_is_noteworthy(entry) == 0)) {
+		    (cm_check_expiration_is_noteworthy(entry,
+						       &cm_prefs_notify_ttls,
+						       &entry->cm_last_need_notify_check) == 0)) {
 			/* Kick off a notification. */
 			entry->cm_state = CM_NEED_TO_NOTIFY;
 			*when = cm_time_now;
+		} else
+		if (entry->cm_autorenew &&
+		    (cm_check_expiration_is_noteworthy(entry,
+						       &cm_prefs_enroll_ttls,
+						       &entry->cm_last_need_enroll_check) == 0)) {
+			/* Kick off an enrollment attempt.  We need to go all
+			 * the way back to generating the CSR because the user
+			 * may have asked us to request with parameters that
+			 * have changed since we last generated a CSR. */
+			entry->cm_state = CM_NEED_CSR;
+			*when = cm_time_now;
 		} else {
-			/* Nothing to do here.  Check again tomorrow. */
+			/* Nothing to do here.  Check again at an appropriate time. */
 			*when = cm_time_delay;
 			*delay = cm_decide_monitor_delay(remaining);
 		}
@@ -907,16 +984,20 @@ cm_iterate(struct cm_store_entry *entry, struct cm_store_ca *ca,
 		if (cm_notify_ready(entry, state->cm_notify_state) == 0) {
 			cm_notify_done(entry, state->cm_notify_state);
 			state->cm_notify_state = NULL;
-			if (entry->cm_autorenew) {
-				/* We need to go all the way back to generating
-				 * the CSR because the user may have asked us
-				 * to request with parameters that have changed
-				 * since we last generated a CSR. */
+			if (entry->cm_autorenew &&
+			    (cm_check_expiration_is_noteworthy(entry,
+							       &cm_prefs_enroll_ttls,
+							       &entry->cm_last_need_enroll_check) == 0)) {
+				/* Kick off an enrollment attempt.  We need to go all
+				 * the way back to generating the CSR because the user
+				 * may have asked us to request with parameters that
+				 * have changed since we last generated a CSR. */
 				entry->cm_state = CM_NEED_CSR;
 				*when = cm_time_now;
 			} else {
 				entry->cm_state = CM_MONITORING;
-				*when = cm_time_now;
+				*when = cm_time_delay;
+				*delay = cm_decide_monitor_delay(-1);
 			}
 		} else {
 			/* Wait for status update, or poll. */
