@@ -54,6 +54,7 @@ main(int argc, char **argv)
 	struct cm_context *ctx;
 	enum cm_tdbus_type bus;
 	int i, c, dlevel = 0, pfd = -1, lfd = -1;
+	long l;
 	pid_t pid;
 	FILE *pfp;
 	const char *pidfile = NULL, *tmpdir;
@@ -110,17 +111,18 @@ main(int argc, char **argv)
 			dofork = FALSE;
 			break;
 		default:
-			printf("Usage: %s [-s|-S] [-n|-f] [-d LEVEL] "
-			       "[-p FILE]\n"
-			       "\t-s         use session bus\n"
-			       "\t-S         use system bus\n"
-			       "\t-n         don't become a daemon\n"
-			       "\t-f         do become a daemon\n"
-			       "\t-b TIMEOUT bus-activated, idle timeout\n"
-			       "\t-B         don't use an idle timeout\n"
-			       "\t-d LEVEL   set debugging level (implies -n)\n"
-			       "\t-p FILE    write service PID to file\n",
+			printf(_("Usage: %s [-s|-S] [-n|-f] [-d LEVEL] "
+			         "[-p FILE]\n"),
 			       cm_env_whoami());
+			printf("%s%s%s%s%s%s%s%s",
+			       _("\t-s         use session bus\n"),
+			       _("\t-S         use system bus\n"),
+			       _("\t-n         don't become a daemon\n"),
+			       _("\t-f         do become a daemon\n"),
+			       _("\t-b TIMEOUT bus-activated, idle timeout\n"),
+			       _("\t-B         don't use an idle timeout\n"),
+			       _("\t-d LEVEL   set debugging level (implies -n)\n"),
+			       _("\t-p FILE    write service PID to file\n"));
 			exit(1);
 			break;
 		}
@@ -173,6 +175,10 @@ main(int argc, char **argv)
 			close(lfd);
 			exit(1);
 		}
+		l = fcntl(lfd, F_GETFD);
+		if (l != -1) {
+			fcntl(lfd, F_SETFD, l | FD_CLOEXEC);
+		}
 		break;
 	}
 
@@ -214,6 +220,10 @@ main(int argc, char **argv)
 			close(pfd);
 			exit(1);
 		}
+		l = fcntl(pfd, F_GETFD);
+		if (l != -1) {
+			fcntl(pfd, F_SETFD, l | FD_CLOEXEC);
+		}
 		pfp = fdopen(pfd, "w");
 		if (pfp == NULL) {
 			fprintf(stderr, "Error opening pidfile \"%s\": %s\n",
