@@ -76,6 +76,8 @@ enum cm_store_file_field {
 	cm_store_entry_field_cert_eku,
 
 	cm_store_entry_field_last_expiration_check,
+	cm_store_entry_field_last_need_notify_check,
+	cm_store_entry_field_last_need_enroll_check,
 
 	cm_store_entry_field_template_subject,
 	cm_store_entry_field_template_hostname,
@@ -102,6 +104,8 @@ enum cm_store_file_field {
 
 	cm_store_entry_field_cert,
 
+	cm_store_entry_field_pre_certsave_command,
+	cm_store_entry_field_pre_certsave_uid,
 	cm_store_entry_field_post_certsave_command,
 	cm_store_entry_field_post_certsave_uid,
 
@@ -154,6 +158,8 @@ static struct cm_store_file_field_list {
 	{cm_store_entry_field_cert_eku, "cert_eku"},
 
 	{cm_store_entry_field_last_expiration_check, "last_expiration_check"},
+	{cm_store_entry_field_last_need_notify_check, "last_need_notify_check"},
+	{cm_store_entry_field_last_need_enroll_check, "last_need_enroll_check"},
 
 	{cm_store_entry_field_template_subject, "template_subject"},
 	{cm_store_entry_field_template_hostname, "template_hostname"},
@@ -178,6 +184,8 @@ static struct cm_store_file_field_list {
 	{cm_store_entry_field_ca_error, "ca_error"},
 
 	{cm_store_entry_field_cert, "cert"},
+	{cm_store_entry_field_pre_certsave_command, "pre_certsave_command"},
+	{cm_store_entry_field_pre_certsave_uid, "pre_certsave_uid"},
 	{cm_store_entry_field_post_certsave_command, "post_certsave_command"},
 	{cm_store_entry_field_post_certsave_uid, "post_certsave_uid"},
 
@@ -549,7 +557,21 @@ cm_store_entry_read(void *parent, const char *filename, FILE *fp)
 				ret->cm_cert_eku = free_if_empty(p);
 				break;
 			case cm_store_entry_field_last_expiration_check:
-				ret->cm_last_expiration_check =
+				/* backward compatibility before we split them
+				 * into two settings */
+				ret->cm_last_need_notify_check =
+					cm_store_time_from_timestamp(p);
+				ret->cm_last_need_enroll_check =
+					cm_store_time_from_timestamp(p);
+				talloc_free(p);
+				break;
+			case cm_store_entry_field_last_need_notify_check:
+				ret->cm_last_need_notify_check =
+					cm_store_time_from_timestamp(p);
+				talloc_free(p);
+				break;
+			case cm_store_entry_field_last_need_enroll_check:
+				ret->cm_last_need_enroll_check =
 					cm_store_time_from_timestamp(p);
 				talloc_free(p);
 				break;
@@ -612,6 +634,12 @@ cm_store_entry_read(void *parent, const char *filename, FILE *fp)
 			case cm_store_entry_field_cert:
 				ret->cm_cert = free_if_empty(p);
 				break;
+			case cm_store_entry_field_pre_certsave_command:
+				ret->cm_pre_certsave_command  = free_if_empty(p);
+				break;
+			case cm_store_entry_field_pre_certsave_uid:
+				ret->cm_pre_certsave_uid = free_if_empty(p);
+				break;
 			case cm_store_entry_field_post_certsave_command:
 				ret->cm_post_certsave_command  = free_if_empty(p);
 				break;
@@ -696,6 +724,8 @@ cm_store_ca_read(void *parent, const char *filename, FILE *fp)
 			case cm_store_entry_field_cert_ku:
 			case cm_store_entry_field_cert_eku:
 			case cm_store_entry_field_last_expiration_check:
+			case cm_store_entry_field_last_need_notify_check:
+			case cm_store_entry_field_last_need_enroll_check:
 			case cm_store_entry_field_template_subject:
 			case cm_store_entry_field_template_hostname:
 			case cm_store_entry_field_template_email:
@@ -713,6 +743,8 @@ cm_store_ca_read(void *parent, const char *filename, FILE *fp)
 			case cm_store_entry_field_ca_cookie:
 			case cm_store_entry_field_ca_error:
 			case cm_store_entry_field_cert:
+			case cm_store_entry_field_pre_certsave_command:
+			case cm_store_entry_field_pre_certsave_uid:
 			case cm_store_entry_field_post_certsave_command:
 			case cm_store_entry_field_post_certsave_uid:
 				break;
@@ -959,10 +991,12 @@ cm_store_entry_write(FILE *fp, struct cm_store_entry *entry)
 	cm_store_file_write_str(fp, cm_store_entry_field_cert_eku,
 				entry->cm_cert_eku);
 
-	cm_store_file_write_str(fp, cm_store_entry_field_last_expiration_check,
-				cm_store_timestamp_from_time(entry->cm_last_expiration_check,
+	cm_store_file_write_str(fp, cm_store_entry_field_last_need_notify_check,
+				cm_store_timestamp_from_time(entry->cm_last_need_notify_check,
+							     timestamp));
+	cm_store_file_write_str(fp, cm_store_entry_field_last_need_enroll_check,
+				cm_store_timestamp_from_time(entry->cm_last_need_enroll_check,
 							     timestamp));
-
 	cm_store_file_write_str(fp, cm_store_entry_field_template_subject,
 				entry->cm_template_subject);
 	cm_store_file_write_strs(fp, cm_store_entry_field_template_hostname,
@@ -1002,6 +1036,10 @@ cm_store_entry_write(FILE *fp, struct cm_store_entry *entry)
 	cm_store_file_write_str(fp, cm_store_entry_field_ca_error,
 				entry->cm_ca_error);
 	cm_store_file_write_str(fp, cm_store_entry_field_cert, entry->cm_cert);
+	cm_store_file_write_str(fp, cm_store_entry_field_pre_certsave_command,
+				entry->cm_pre_certsave_command);
+	cm_store_file_write_str(fp, cm_store_entry_field_pre_certsave_uid,
+				entry->cm_pre_certsave_uid);
 	cm_store_file_write_str(fp, cm_store_entry_field_post_certsave_command,
 				entry->cm_post_certsave_command);
 	cm_store_file_write_str(fp, cm_store_entry_field_post_certsave_uid,
@@ -1491,7 +1529,8 @@ cm_store_entry_dup(void *parent, struct cm_store_entry *entry)
 	ret->cm_cert_ku = cm_store_maybe_strdup(ret, entry->cm_cert_ku);
 	ret->cm_cert_eku = cm_store_maybe_strdup(ret, entry->cm_cert_eku);
 
-	ret->cm_last_expiration_check = entry->cm_last_expiration_check;
+	ret->cm_last_need_notify_check = entry->cm_last_need_notify_check;
+	ret->cm_last_need_enroll_check = entry->cm_last_need_enroll_check;
 	ret->cm_notification_method = entry->cm_notification_method;
 	ret->cm_notification_destination = cm_store_maybe_strdup(ret, entry->cm_notification_destination);
 
@@ -1513,6 +1552,8 @@ cm_store_entry_dup(void *parent, struct cm_store_entry *entry)
 	ret->cm_ca_cookie = cm_store_maybe_strdup(ret, entry->cm_ca_cookie);
 	ret->cm_ca_error = cm_store_maybe_strdup(ret, entry->cm_ca_error);
 	ret->cm_cert = cm_store_maybe_strdup(ret, entry->cm_cert);
+	ret->cm_pre_certsave_command = cm_store_maybe_strdup(ret, entry->cm_pre_certsave_command);
+	ret->cm_pre_certsave_uid = cm_store_maybe_strdup(ret, entry->cm_pre_certsave_uid);
 	ret->cm_post_certsave_command = cm_store_maybe_strdup(ret, entry->cm_post_certsave_command);
 	ret->cm_post_certsave_uid = cm_store_maybe_strdup(ret, entry->cm_post_certsave_uid);
 
