@@ -57,6 +57,8 @@ static struct {
 	{"CA_REJECTED", CM_CA_REJECTED},
 	{"CA_WORKING", CM_CA_WORKING},
 	{"NEED_TO_SAVE_CERT", CM_NEED_TO_SAVE_CERT},
+	{"PRE_SAVE_CERT", CM_PRE_SAVE_CERT},
+	{"START_SAVING_CERT", CM_START_SAVING_CERT},
 	{"SAVING_CERT", CM_SAVING_CERT},
 	{"NEED_TO_READ_CERT", CM_NEED_TO_READ_CERT},
 	{"READING_CERT", CM_READING_CERT},
