@@ -70,13 +70,15 @@ struct cm_store_entry {
 	char **cm_cert_principal;
 	char *cm_cert_ku;
 	char *cm_cert_eku;
-	time_t cm_last_expiration_check;
+	time_t cm_last_need_notify_check;
+	time_t cm_last_need_enroll_check;
 	/* How to notify administrator: syslog(LOG_AUTHPRIV?), mail to root@? */
 	enum cm_notification_method {
 		cm_notification_unspecified,
 		cm_notification_none,
 		cm_notification_syslog,
 		cm_notification_email,
+		cm_notification_command,
 		cm_notification_stdout,	/* for testing _ONLY_ */
 	} cm_notification_method;
 	char *cm_notification_destination;
@@ -112,7 +114,8 @@ struct cm_store_entry {
 		CM_HAVE_CSR, CM_NEED_TO_SUBMIT, CM_SUBMITTING,
 		CM_NEED_CA, CM_CA_UNREACHABLE, CM_CA_UNCONFIGURED,
 		CM_CA_REJECTED, CM_CA_WORKING,
-		CM_NEED_TO_SAVE_CERT, CM_SAVING_CERT,
+		CM_NEED_TO_SAVE_CERT, CM_PRE_SAVE_CERT,
+		CM_START_SAVING_CERT, CM_SAVING_CERT,
 		CM_NEED_TO_READ_CERT, CM_READING_CERT,
 		CM_SAVED_CERT, CM_POST_SAVED_CERT,
 		CM_MONITORING, CM_NEED_TO_NOTIFY, CM_NOTIFYING,
@@ -140,6 +143,10 @@ struct cm_store_entry {
 	char *cm_ca_error;
 	/* The certificate, if we have one. */
 	char *cm_cert;
+	/* A command to run before we save the certificate. */
+	char *cm_pre_certsave_command;
+	/* The UID of the user as whom we run the above command. */
+	char *cm_pre_certsave_uid;
 	/* A command to run after we save the certificate. */
 	char *cm_post_certsave_command;
 	/* The UID of the user as whom we run the above command. */
