@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009,2011 Red Hat, Inc.
+ * Copyright (C) 2009,2011,2012 Red Hat, Inc.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,6 +24,10 @@ struct cm_store_ca;
 /* Generic routines. */
 struct cm_store_entry *cm_store_entry_new(void *parent);
 struct cm_store_ca *cm_store_ca_new(void *parent);
+struct cm_store_entry *cm_store_entry_dup(void *parent,
+					  struct cm_store_entry *entry);
+struct cm_store_ca *cm_store_ca_dup(void *parent,
+				    struct cm_store_ca *ca);
 
 /* Store-specific entry storage. */
 int cm_store_entry_save(struct cm_store_entry *entry);
@@ -40,13 +44,15 @@ time_t cm_store_time_from_timestamp(const char *timestamp);
 char *cm_store_timestamp_from_time(time_t when, char timestamp[15]);
 char *cm_store_timestamp_from_time_for_display(time_t when, char timestamp[24]);
 char *cm_store_increment_serial(void *parent, const char *old_serial);
-char *cm_store_serial_from_binary(void *parent,
-				  const unsigned char *serial, int length);
 char *cm_store_serial_to_binary(void *parent,
 				const unsigned char *serial, int length);
 char *cm_store_serial_to_der(void *parent, const char *serial);
+char *cm_store_hex_from_bin(void *parent,
+			    const unsigned char *serial, int length);
 void cm_store_hex_to_bin(const char *serial, unsigned char *buf, int length);
 char *cm_store_canonicalize_directory(void *parent, const char *path);
+char *cm_store_maybe_strdup(void *parent, const char *s);
+char **cm_store_maybe_strdupv(void *parent, char **s);
 
 void cm_store_set_if_not_set_s(void *parent, char **dest, char *src);
 void cm_store_set_if_not_set_as(void *parent, char ***dest, char **src);
