@@ -27,16 +27,25 @@ enum cm_submit_h_opt_delegate {
 	cm_submit_h_delegate_off,
 	cm_submit_h_delegate_on
 };
+enum cm_submit_h_opt_clientauth {
+	cm_submit_h_clientauth_off,
+	cm_submit_h_clientauth_on
+};
 struct cm_submit_h_context *cm_submit_h_init(void *parent,
 					     const char *method,
 					     const char *uri,
 					     const char *args,
 					     const char *cainfo,
 					     const char *capath,
+					     const char *sslcert,
+					     const char *sslkey,
+					     const char *sslpass,
 					     enum cm_submit_h_opt_negotiate neg,
-					     enum cm_submit_h_opt_delegate del);
+					     enum cm_submit_h_opt_delegate del,
+					     enum cm_submit_h_opt_clientauth cli);
 void cm_submit_h_run(struct cm_submit_h_context *ctx);
 int cm_submit_h_result_code(struct cm_submit_h_context *ctx);
+const char *cm_submit_h_result_code_text(struct cm_submit_h_context *ctx);
 const char *cm_submit_h_results(struct cm_submit_h_context *ctx);
 
 #endif
