@@ -39,6 +39,8 @@
 #define CM_DBUS_PROP_NICKNAME "nickname"
 #define CM_DBUS_PROP_AUTORENEW "autorenew"
 #define CM_DBUS_PROP_CERT "cert"
+#define CM_DBUS_PROP_CERT_PRESAVE_COMMAND "cert-presave-command"
+#define CM_DBUS_PROP_CERT_PRESAVE_UID "cert-presave-uid"
 #define CM_DBUS_PROP_CERT_POSTSAVE_COMMAND "cert-postsave-command"
 #define CM_DBUS_PROP_CERT_POSTSAVE_UID "cert-postsave-uid"
 #define CM_DBUS_PROP_CERT_ISSUER "issuer"
@@ -71,6 +73,7 @@
 #define CM_DBUS_PROP_NOTIFICATION_TYPE "notification-type"
 #define CM_DBUS_PROP_NOTIFICATION_SYSLOG_PRIORITY "notification-syslog-priority"
 #define CM_DBUS_PROP_NOTIFICATION_EMAIL "notification-email"
+#define CM_DBUS_PROP_NOTIFICATION_COMMAND "notification-command"
 #define CM_DBUS_PROP_KEY_PIN_FILE "key-pin-file"
 #define CM_DBUS_PROP_KEY_PIN "key-pin"
 #define CM_DBUS_PROP_STATUS "status"
